from distutils.core import setup
setup(
  name = 'sopy_fem',
  packages = ['sopy_fem'], # this must be the same as the name above
  version = '0.0.1',
  description = 'An academic program for finite elments analysis of 2D solids and structures',
  author = 'Daniel Di Capua',
  author_email = 'dicapua67@gmail.com',
  url = 'https://github.com/DanielDiCapua/sopy_fem.git', # use the URL to the github repo
  download_url = 'https://github.com/DanielDiCapua/sopy_fem.git/tarball/0.0.1',
  keywords = ['developing', 'example'],
  classifiers = [],
)